@@ -53,12 +53,36 @@
 
 using namespace KNSCore;
 
+class EnginePrivate {
+public:
+    QList<Provider::CategoryMetadata> categoriesMetadata;
+};
+
+// BCI: Add a real d-pointer
+typedef QHash<const Engine *, EnginePrivate *> EnginePrivateHash;
+Q_GLOBAL_STATIC(EnginePrivateHash, d_func)
+static EnginePrivate *d(const Engine* engine)
+{
+    EnginePrivate* ret = d_func()->value(engine);
+    if (!ret) {
+        ret = new EnginePrivate;
+        d_func()->insert(engine, ret);
+    }
+    return ret;
+}
+static void delete_d(const Engine* engine)
+{
+    EnginePrivate* ret = d_func()->value(engine);
+    delete ret;
+    d_func()->remove(engine);
+}
+
 Engine::Engine(QObject *parent)
     : QObject(parent)
     , m_installation(new Installation)
-    , m_cache(0)
+    , m_cache()
     , m_searchTimer(new QTimer)
-    , m_atticaProviderManager(0)
+    , m_atticaProviderManager(nullptr)
     , m_currentPage(-1)
     , m_pageSize(20)
     , m_numDataJobs(0)
@@ -71,9 +95,6 @@ Engine::Engine(QObject *parent)
     connect(m_searchTimer, &QTimer::timeout, this, &Engine::slotSearchTimerExpired);
     connect(m_installation, &Installation::signalInstallationFinished, this, &Engine::slotInstallationFinished);
     connect(m_installation, &Installation::signalInstallationFailed, this, &Engine::slotInstallationFailed);
-    connect(m_installation, &Installation::signalInformation, this, &Engine::signalMessage);
-    connect(m_installation, &Installation::signalError, this, &Engine::signalError);
-
 }
 
 Engine::~Engine()
@@ -84,6 +105,7 @@ Engine::~Engine()
     delete m_atticaProviderManager;
     delete m_searchTimer;
     delete m_installation;
+    delete_d(this);
 }
 
 bool Engine::init(const QString &configfile)
@@ -113,6 +135,7 @@ bool Engine::init(const QString &configfile)
     }
 
     m_categories = group.readEntry("Categories", QStringList());
+    m_adoptionCommand = group.readEntry("AdoptionCommand", QString());
 
     qCDebug(KNEWSTUFFCORE) << "Categories: " << m_categories;
     m_providerFileUrl = group.readEntry("ProvidersUrl", QString());
@@ -147,6 +170,11 @@ QStringList Engine::categoriesFilter() const
     return m_currentRequest.categories;
 }
 
+QList<Provider::CategoryMetadata> Engine::categoriesMetadata()
+{
+    return d(this)->categoriesMetadata;
+}
+
 void Engine::loadProviders()
 {
     if (m_providerFileUrl.isEmpty()) {
@@ -192,6 +220,11 @@ void Engine::slotProviderFileLoaded(const QDomDocument &doc)
         QSharedPointer<KNSCore::Provider> provider;
         if (isAtticaProviderFile || n.attribute(QStringLiteral("type")).toLower() == QLatin1String("rest")) {
             provider = QSharedPointer<KNSCore::Provider> (new AtticaProvider(m_categories));
+            connect(provider.data(), &Provider::categoriesMetadataLoded,
+                    this, [this](const QList<Provider::CategoryMetadata> &categories){
+                        d(this)->categoriesMetadata = categories;
+                        emit signalCategoriesMetadataLoded(categories);
+                    });
         } else {
             provider = QSharedPointer<KNSCore::Provider> (new StaticXmlProvider);
         }
@@ -215,6 +248,11 @@ void Engine::atticaProviderLoaded(const Attica::Provider &atticaProvider)
     }
     QSharedPointer<KNSCore::Provider> provider =
         QSharedPointer<KNSCore::Provider> (new AtticaProvider(atticaProvider, m_categories));
+    connect(provider.data(), &Provider::categoriesMetadataLoded,
+            this, [this](const QList<Provider::CategoryMetadata> &categories){
+                d(this)->categoriesMetadata = categories;
+                emit signalCategoriesMetadataLoded(categories);
+            });
     addProvider(provider);
 }
 
@@ -223,8 +261,7 @@ void Engine::addProvider(QSharedPointer<KNSCore::Provider> provider)
     qCDebug(KNEWSTUFFCORE) << "Engine addProvider called with provider with id " << provider->id();
     m_providers.insert(provider->id(), provider);
     connect(provider.data(), &Provider::providerInitialized, this, &Engine::providerInitialized);
-    connect(provider.data(), SIGNAL(loadingFinished(KNSCore::Provider::SearchRequest,KNSCore::EntryInternal::List)),
-            SLOT(slotEntriesLoaded(KNSCore::Provider::SearchRequest,KNSCore::EntryInternal::List)));
+    connect(provider.data(), &Provider::loadingFinished, this, &Engine::slotEntriesLoaded);
     connect(provider.data(), &Provider::entryDetailsLoaded, this, &Engine::slotEntryDetailsLoaded);
     connect(provider.data(), &Provider::payloadLinkLoaded, this, &Engine::downloadLinkLoaded);
     connect(provider.data(), &Provider::signalError, this, &Engine::signalError);
@@ -568,3 +605,57 @@ void KNSCore::Engine::checkForInstalled()
         p->loadEntries(request);
     }
 }
+
+/**
+ * we look for the directory where all the resources got installed.
+ * assuming it was extracted into a directory
+ */
+static QDir sharedDir(QStringList dirs, const QString &rootPath)
+{
+    while(!dirs.isEmpty()) {
+        const QString currentPath = QDir::cleanPath(dirs.takeLast());
+        if (!currentPath.startsWith(rootPath))
+            continue;
+
+        const QFileInfo current(currentPath);
+        if (!current.isDir())
+            continue;
+
+        const QDir dir = current.dir();
+        if (dir.path()==(rootPath+dir.dirName())) {
+            return dir;
+        }
+    }
+    return {};
+}
+
+QString Engine::adoptionCommand(const KNSCore::EntryInternal& entry) const
+{
+    auto adoption = m_adoptionCommand;
+    if(adoption.isEmpty())
+        return {};
+
+    const QLatin1String dirReplace("%d");
+    if (adoption.contains(dirReplace)) {
+        QString installPath = sharedDir(entry.installedFiles(), m_installation->targetInstallationPath()).path();
+        adoption.replace(dirReplace, installPath);
+    }
+
+    const QLatin1String fileReplace("%f");
+    QStringList ret;
+    if (adoption.contains(fileReplace)) {
+        if (entry.installedFiles().isEmpty()) {
+            qCWarning(KNEWSTUFFCORE) << "no installed files to adopt";
+        } else if (entry.installedFiles().count() != 1) {
+            qCWarning(KNEWSTUFFCORE) << "can only adopt one file, will be using the first" << entry.installedFiles().at(0);
+        }
+
+        adoption.replace(fileReplace, entry.installedFiles().at(0));
+    }
+    return adoption;
+}
+
+bool KNSCore::Engine::hasAdoptionCommand() const
+{
+    return !m_adoptionCommand.isEmpty();
+}
