@@ -87,15 +87,17 @@ KMoreToolsService* KMoreToolsPresets::registerServiceByDesktopEntryName(KMoreToo
 
 #undef ADD_ENTRY
 
-    auto iter = dict.find(desktopEntryName);
-    if (iter != dict.end()) {
+    auto iter = dict.constFind(desktopEntryName);
+    if (iter != dict.constEnd()) {
         auto kmtServiceInfo = *iter;
         const QString subdir = QStringLiteral("presets-kmoretools");
         auto serviceLocatingMode = desktopEntryName.endsWith(QLatin1String(".kmt-edition")) ?
                                    KMoreTools::ServiceLocatingMode_ByProvidedExecLine : KMoreTools::ServiceLocatingMode_Default;
         auto service = kmt->registerServiceByDesktopEntryName(desktopEntryName, subdir, serviceLocatingMode);
-        service->setHomepageUrl(QUrl(kmtServiceInfo.homepageUrl));
-        service->setMaxUrlArgCount(kmtServiceInfo.maxUrlArgCount);
+        if (service) { // We might get nullptr in case of missing or broken .desktop files
+            service->setHomepageUrl(QUrl(kmtServiceInfo.homepageUrl));
+            service->setMaxUrlArgCount(kmtServiceInfo.maxUrlArgCount);
+        }
         return service;
     } else {
         qDebug() << "KMoreToolsPresets::registerServiceByDesktopEntryName: " << desktopEntryName << "was not found. Return nullptr.";
@@ -147,10 +149,10 @@ QList<KMoreToolsService*> KMoreToolsPresetsPrivate::registerServicesByGroupingNa
     QSet<QString> alreadyUsedDesktopEntryNames; // including the "more:" keyword
     bool nextIsMore = false;
 
-    Q_FOREACH (QString groupingName, groupingNames) {
-        auto iter = dict.find(groupingName);
-        if (iter != dict.end()) {
-            Q_FOREACH(QString desktopEntryName, *iter) {
+    Q_FOREACH (const QString &groupingName, groupingNames) {
+        auto iter = dict.constFind(groupingName);
+        if (iter != dict.constEnd()) {
+            Q_FOREACH(const QString &desktopEntryName, *iter) {
                 if (!alreadyUsedDesktopEntryNames.contains(desktopEntryName)) {
                     if (desktopEntryName == _("more:")) {
                         nextIsMore = true;
@@ -159,7 +161,10 @@ QList<KMoreToolsService*> KMoreToolsPresetsPrivate::registerServicesByGroupingNa
                             *firstMoreSectionDesktopEntryName = desktopEntryName;
                             nextIsMore = false;
                         }
-                        resultList << KMoreToolsPresets::registerServiceByDesktopEntryName(kmt, desktopEntryName);
+                        KMoreToolsService *kmtService = KMoreToolsPresets::registerServiceByDesktopEntryName(kmt, desktopEntryName);
+                        if (kmtService) { // Do not add null pointers caused by missing or broken .desktop files
+                            resultList << kmtService;
+                        }
                     }
                 } else {
                     alreadyUsedDesktopEntryNames.insert(desktopEntryName);
